@@ -1,5 +1,5 @@
 package Git::Raw::Tag;
-$Git::Raw::Tag::VERSION = '0.44';
+$Git::Raw::Tag::VERSION = '0.45';
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Git::Raw::Tag - Git tag class
 
 =head1 VERSION
 
-version 0.44
+version 0.45
 
 =head1 SYNOPSIS
 
@@ -57,10 +57,13 @@ doesn't exist, this function wil return C<undef>.
 
 Retrieve the L<Git::Raw::Repository> owning the tag.
 
-=head2 foreach( $repo, $callback )
+=head2 foreach( $repo, $callback, [$type] )
 
-Run C<$callback> for every tag in the repo. The callback receives a tag object.
-A non-zero return value stops the loop.
+Run C<$callback> for every tag in the repo. The callback receives a tag object,
+which will either a be a L<Git::Raw::Tag> object for annotated tags, or a
+L<Git::Raw::Reference> for lightweight tags. C<$type> may be C<"all">,
+C<"annotated"> or C<"lightweight">. If C<$type> is not specified or is C<undef>,
+all tags will be returned. A non-zero return value stops the loop.
 
 =head2 delete( )
 
